@@ -7,6 +7,6 @@ let%expect_test "[Base.Hash.state] is still immediate" =
   [%expect {| |}]
 
 let%expect_test _ =
-  print_s [%sexp (Caml.Obj.is_int (Caml.Obj.repr (Base.Hash.create ~seed:1 ())) : bool)];
+  print_s [%sexp (Stdlib.Obj.is_int (Stdlib.Obj.repr (Base.Hash.create ~seed:1 ())) : bool)];
   [%expect {| true |}];
 ;;
