@@ -71,8 +71,8 @@ val invariant : ('k, 'v) t -> compare:('k -> 'k -> int) -> unit
 val add
   :  ('k, 'v) t
   -> replace:bool
-  -> compare:('k -> 'k -> int)
-  -> added:bool ref
+  -> compare:(('k -> 'k -> int)[@local])
+  -> added:(bool ref[@local])
   -> key:'k
   -> data:'v
   -> ('k, 'v) t
@@ -84,7 +84,7 @@ val last : ('k, 'v) t -> ('k * 'v) option
 
 (** If the specified key exists in the tree, returns the corresponding value.  O(log(N))
     time and O(1) space. *)
-val find : ('k, 'v) t -> compare:('k -> 'k -> int) -> 'k -> 'v option
+val find : ('k, 'v) t -> compare:(('k -> 'k -> int)[@local]) -> 'k -> 'v option
 
 (** [find_and_call t ~compare k ~if_found ~if_not_found]
 
@@ -95,49 +95,82 @@ val find : ('k, 'v) t -> compare:('k -> 'k -> int) -> 'k -> 'v option
     except that it doesn't allocate the option. *)
 val find_and_call
   :  ('k, 'v) t
-  -> compare:('k -> 'k -> int)
+  -> compare:(('k -> 'k -> int)[@local])
   -> 'k
-  -> if_found:('v -> 'a)
-  -> if_not_found:('k -> 'a)
+  -> if_found:(('v -> 'a)[@local])
+  -> if_not_found:(('k -> 'a)[@local])
   -> 'a
 
 val find_and_call1
   :  ('k, 'v) t
-  -> compare:('k -> 'k -> int)
+  -> compare:(('k -> 'k -> int)[@local])
   -> 'k
-  -> 'a
-  -> if_found:('v -> 'a -> 'b)
-  -> if_not_found:('k -> 'a -> 'b)
+  -> a:'a
+  -> if_found:(('v -> 'a -> 'b)[@local])
+  -> if_not_found:(('k -> 'a -> 'b)[@local])
   -> 'b
 
+val find_and_call2
+  :  ('k, 'v) t
+  -> compare:(('k -> 'k -> int)[@local])
+  -> 'k
+  -> a:'a
+  -> b:'b
+  -> if_found:(('v -> 'a -> 'b -> 'c)[@local])
+  -> if_not_found:(('k -> 'a -> 'b -> 'c)[@local])
+  -> 'c
+
 val findi_and_call
   :  ('k, 'v) t
-  -> compare:('k -> 'k -> int)
+  -> compare:(('k -> 'k -> int)[@local])
   -> 'k
-  -> if_found:(key:'k -> data:'v -> 'a)
-  -> if_not_found:('k -> 'a)
+  -> if_found:((key:'k -> data:'v -> 'a)[@local])
+  -> if_not_found:(('k -> 'a)[@local])
   -> 'a
 
+val findi_and_call1
+  :  ('k, 'v) t
+  -> compare:(('k -> 'k -> int)[@local])
+  -> 'k
+  -> a:'a
+  -> if_found:((key:'k -> data:'v -> 'a -> 'b)[@local])
+  -> if_not_found:(('k -> 'a -> 'b)[@local])
+  -> 'b
+
+val findi_and_call2
+  :  ('k, 'v) t
+  -> compare:(('k -> 'k -> int)[@local])
+  -> 'k
+  -> a:'a
+  -> b:'b
+  -> if_found:((key:'k -> data:'v -> 'a -> 'b -> 'c)[@local])
+  -> if_not_found:(('k -> 'a -> 'b -> 'c)[@local])
+  -> 'c
+
 (** Returns true if key is present in the tree, and false otherwise. *)
-val mem : ('k, 'v) t -> compare:('k -> 'k -> int) -> 'k -> bool
+val mem : ('k, 'v) t -> compare:(('k -> 'k -> int)[@local]) -> 'k -> bool
 
 (** Removes key destructively from the tree if it exists, returning the new root node.
     Previous root nodes are not usable anymore; do so at your peril. The [removed] ref
     will be set to true if a node was actually removed, and false otherwise. *)
 val remove
   :  ('k, 'v) t
-  -> removed:bool ref
-  -> compare:('k -> 'k -> int)
+  -> removed:(bool ref[@local])
+  -> compare:(('k -> 'k -> int)[@local])
   -> 'k
   -> ('k, 'v) t
 
 (** Folds over the tree. *)
-val fold : ('k, 'v) t -> init:'a -> f:(key:'k -> data:'v -> 'a -> 'a) -> 'a
+val fold
+  :  ('k, 'v) t
+  -> init:'acc
+  -> f:((key:'k -> data:'v -> 'acc -> 'acc)[@local])
+  -> 'acc
 
 (** Iterates over the tree. *)
-val iter : ('k, 'v) t -> f:(key:'k -> data:'v -> unit) -> unit
+val iter : ('k, 'v) t -> f:((key:'k -> data:'v -> unit)[@local]) -> unit
 
 (** Map over the the tree, changing the data in place. *)
-val mapi_inplace : ('k, 'v) t -> f:(key:'k -> data:'v -> 'v) -> unit
+val mapi_inplace : ('k, 'v) t -> f:((key:'k -> data:'v -> 'v)[@local]) -> unit
 
 val choose_exn : ('k, 'v) t -> 'k * 'v
