@@ -4,15 +4,23 @@ module String = String0
 include Char0
 
 module T = struct
-  type t = char [@@deriving_inline compare, hash, sexp]
+  type t = char [@@deriving_inline compare, hash, globalize, sexp, sexp_grammar]
+
   let compare = (compare_char : t -> t -> int)
-  let (hash_fold_t :
-         Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
+
+  let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
     hash_fold_char
+
   and (hash : t -> Ppx_hash_lib.Std.Hash.hash_value) =
-    let func = hash_char in fun x -> func x
-  let t_of_sexp = (char_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
-  let sexp_of_t = (sexp_of_char : t -> Ppx_sexp_conv_lib.Sexp.t)
+    let func = hash_char in
+    fun x -> func x
+  ;;
+
+  let (globalize : (t[@ocaml.local]) -> t) = (globalize_char : (t[@ocaml.local]) -> t)
+  let t_of_sexp = (char_of_sexp : Sexplib0.Sexp.t -> t)
+  let sexp_of_t = (sexp_of_char : t -> Sexplib0.Sexp.t)
+  let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = char_sexp_grammar
+
   [@@@end]
 
   let to_string t = String.make 1 t
@@ -32,11 +40,14 @@ include Identifiable.Make (struct
     let module_name = "Base.Char"
   end)
 
+let pp fmt c = Stdlib.Format.fprintf fmt "%C" c
+
 (* Open replace_polymorphic_compare after including functor instantiations so they do not
    shadow its definitions. This is here so that efficient versions of the comparison
    functions are available within this module. *)
 open! Char_replace_polymorphic_compare
 
+let invariant (_ : t) = ()
 let all = Array.init 256 ~f:unsafe_of_int |> Array.to_list
 
 let is_lowercase = function
@@ -86,6 +97,34 @@ let get_digit_exn t =
 
 let get_digit t = if is_digit t then Some (get_digit_unsafe t) else None
 
+let is_hex_digit = function
+  | '0' .. '9' | 'a' .. 'f' | 'A' .. 'F' -> true
+  | _ -> false
+;;
+
+let is_hex_digit_lower = function
+  | '0' .. '9' | 'a' .. 'f' -> true
+  | _ -> false
+;;
+
+let is_hex_digit_upper = function
+  | '0' .. '9' | 'A' .. 'F' -> true
+  | _ -> false
+;;
+
+let get_hex_digit_exn = function
+  | '0' .. '9' as t -> to_int t - to_int '0'
+  | 'a' .. 'f' as t -> to_int t - to_int 'a' + 10
+  | 'A' .. 'F' as t -> to_int t - to_int 'A' + 10
+  | t ->
+    Error.raise_s
+      (Sexp.message
+         "Char.get_hex_digit_exn: not a hexadecimal digit"
+         [ "char", sexp_of_t t ])
+;;
+
+let get_hex_digit t = if is_hex_digit t then Some (get_hex_digit_exn t) else None
+
 module O = struct
   let ( >= ) = ( >= )
   let ( <= ) = ( <= )
@@ -95,6 +134,25 @@ module O = struct
   let ( <> ) = ( <> )
 end
 
+module Caseless = struct
+  module T = struct
+    type t = char [@@deriving_inline sexp, sexp_grammar]
+
+    let t_of_sexp = (char_of_sexp : Sexplib0.Sexp.t -> t)
+    let sexp_of_t = (sexp_of_char : t -> Sexplib0.Sexp.t)
+    let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = char_sexp_grammar
+
+    [@@@end]
+
+    let compare c1 c2 = compare (lowercase c1) (lowercase c2)
+    let hash_fold_t state t = hash_fold_char state (lowercase t)
+    let hash t = Hash.run hash_fold_t t
+  end
+
+  include T
+  include Comparable.Make (T)
+end
+
 (* Include type-specific [Replace_polymorphic_compare] at the end, after
    including functor application that could shadow its definitions. This is
    here so that efficient versions of the comparison functions are exported by
