@@ -3,9 +3,12 @@
 open! Import
 
 (** An alias for the type of characters. *)
-type t = char [@@deriving_inline enumerate, sexp, sexp_grammar]
+type t = char [@@deriving_inline enumerate, globalize, sexp, sexp_grammar]
 
 include Ppx_enumerate_lib.Enumerable.S with type t := t
+
+val globalize : (t[@ocaml.local]) -> t
+
 include Sexplib0.Sexpable.S with type t := t
 
 val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
