@@ -27,6 +27,10 @@ module type S_fc = sig
   include S with type t := comparable_t
 end
 
+module Module = struct
+  type ('a, 'b) t = (module S with type t = 'a and type comparator_witness = 'b)
+end
+
 let make (type t) ~compare ~sexp_of_t =
   (module struct
     type comparable_t = t
@@ -48,12 +52,11 @@ end
 
 module Make (M : sig
     type t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : t -> t -> int
-        val sexp_of_t : t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    include Ppx_compare_lib.Comparable.S with type t := t
+
+    val sexp_of_t : t -> Sexplib0.Sexp.t
+
     [@@@end]
   end) =
 struct
@@ -89,24 +92,22 @@ end
 
 module type Derived = sig
   type 'a t
-  type 'cmp comparator_witness
+  type !'cmp comparator_witness
 
   val comparator : ('a, 'cmp) comparator -> ('a t, 'cmp comparator_witness) comparator
 end
 
 module Derived (M : sig
     type 'a t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
-        val sexp_of_t :
-          ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+
+    val sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
+
     [@@@end]
   end) =
 struct
-  type 'cmp comparator_witness
+  type !'cmp comparator_witness
 
   let comparator a =
     { compare = M.compare a.compare; sexp_of_t = M.sexp_of_t a.sexp_of_t }
@@ -115,7 +116,7 @@ end
 
 module type Derived2 = sig
   type ('a, 'b) t
-  type ('cmp_a, 'cmp_b) comparator_witness
+  type (!'cmp_a, !'cmp_b) comparator_witness
 
   val comparator
     :  ('a, 'cmp_a) comparator
@@ -125,21 +126,19 @@ end
 
 module Derived2 (M : sig
     type ('a, 'b) t [@@deriving_inline compare, sexp_of]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        val compare :
-          ('a -> 'a -> int) ->
-          ('b -> 'b -> int) -> ('a, 'b) t -> ('a, 'b) t -> int
-        val sexp_of_t :
-          ('a -> Ppx_sexp_conv_lib.Sexp.t) ->
-          ('b -> Ppx_sexp_conv_lib.Sexp.t) ->
-          ('a, 'b) t -> Ppx_sexp_conv_lib.Sexp.t
-      end[@@ocaml.doc "@inline"]
+
+    include Ppx_compare_lib.Comparable.S2 with type ('a, 'b) t := ('a, 'b) t
+
+    val sexp_of_t
+      :  ('a -> Sexplib0.Sexp.t)
+      -> ('b -> Sexplib0.Sexp.t)
+      -> ('a, 'b) t
+      -> Sexplib0.Sexp.t
+
     [@@@end]
   end) =
 struct
-  type ('cmp_a, 'cmp_b) comparator_witness
+  type (!'cmp_a, !'cmp_b) comparator_witness
 
   let comparator a b =
     { compare = M.compare a.compare b.compare
@@ -170,3 +169,35 @@ struct
     { compare = M.compare a.compare; sexp_of_t = M.sexp_of_t a.sexp_of_t }
   ;;
 end
+
+module type Derived2_phantom = sig
+  type ('a, 'b, 'c) t
+  type (!'cmp_a, !'cmp_b) comparator_witness
+
+  val comparator
+    :  ('a, 'cmp_a) comparator
+    -> ('b, 'cmp_b) comparator
+    -> (('a, 'b, _) t, ('cmp_a, 'cmp_b) comparator_witness) comparator
+end
+
+module Derived2_phantom (M : sig
+    type ('a, 'b, 'c) t
+
+    val compare
+      :  ('a -> 'a -> int)
+      -> ('b -> 'b -> int)
+      -> ('a, 'b, 'c) t
+      -> ('a, 'b, 'c) t
+      -> int
+
+    val sexp_of_t : ('a -> Sexp.t) -> ('b -> Sexp.t) -> ('a, 'b, _) t -> Sexp.t
+  end) =
+struct
+  type (!'cmp_a, !'cmp_b) comparator_witness
+
+  let comparator a b =
+    { compare = M.compare a.compare b.compare
+    ; sexp_of_t = M.sexp_of_t a.sexp_of_t b.sexp_of_t
+    }
+  ;;
+end
