@@ -2,8 +2,8 @@
     for convenience and compatibility with other libraries.
 
     The [Format] module itself is deprecated in Base. You may refer to it
-    explicitly through [Caml.Format], though you may wish to search for other
+    explicitly through [Stdlib.Format], though you may wish to search for other
     alternatives for constructing pretty-printers using the [Format.formatter]
     type. *)
 
-type t = Caml.Format.formatter
+type t = Stdlib.Format.formatter
