@@ -40,12 +40,18 @@ end = struct
       | Immediate : Immediate.t repr
       | Non_immediate : Non_immediate.t repr
 
+    external transparent_magic : ('a[@local_opt]) -> ('b[@local_opt]) = "%identity"
+
     let repr =
+      (* [Obj.magic] involves opaqueness under Flambda 2 which will inhibit
+         availability of functions defined in this module for later inlining
+         (e.g. into float.ml). As such we explicitly use %identity here. *)
       match Word_size.word_size with
-      | W64 -> (Caml.Obj.magic Immediate : t repr)
-      | W32 -> (Caml.Obj.magic Non_immediate : t repr)
+      | W64 -> (transparent_magic Immediate : t repr)
+      | W32 -> (transparent_magic Non_immediate : t repr)
     ;;
   end
+  [@@inline always]
 end
 
 include Immediate64.Make (Int) (Int63_emul)
@@ -124,6 +130,30 @@ module Overflow_exn = struct
     else diff
   ;;
 
+  let negative_one = of_int (-1)
+  let div_would_overflow t u = t = min_value && u = negative_one
+
+  let ( * ) t u =
+    let product = t * u in
+    if u <> zero && (div_would_overflow product u || product / u <> t)
+    then
+      raise_s
+        (Sexp.message
+           "( * ) overflow"
+           [ "t", sexp_of_t t; "u", sexp_of_t u; "product", sexp_of_t product ])
+    else product
+  ;;
+
+  let ( / ) t u =
+    if div_would_overflow t u
+    then
+      raise_s
+        (Sexp.message
+           "( / ) overflow"
+           [ "t", sexp_of_t t; "u", sexp_of_t u; "product", sexp_of_t (t / u) ])
+    else t / u
+  ;;
+
   let abs t = if t = min_value then failwith "abs overflow" else abs t
   let neg t = if t = min_value then failwith "neg overflow" else neg t
 end
