@@ -21,17 +21,14 @@
 
 open! Import
 
-type 'a t = 'a lazy_t [@@deriving_inline compare, hash, sexp]
+type 'a t = 'a lazy_t [@@deriving_inline compare, equal, hash, sexp, sexp_grammar]
 
-val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
+include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
+include Ppx_hash_lib.Hashable.S1 with type 'a t := 'a t
+include Sexplib0.Sexpable.S1 with type 'a t := 'a t
 
-val hash_fold_t
-  :  (Ppx_hash_lib.Std.Hash.state -> 'a -> Ppx_hash_lib.Std.Hash.state)
-  -> Ppx_hash_lib.Std.Hash.state
-  -> 'a t
-  -> Ppx_hash_lib.Std.Hash.state
-
-include Ppx_sexp_conv_lib.Sexpable.S1 with type 'a t := 'a t
+val t_sexp_grammar : 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_grammar.t
 
 [@@@end]
 
@@ -43,7 +40,7 @@ exception Undefined
     forced, [Lazy.force x] returns the same value again without recomputing it. If it
     raised an exception, the same exception is raised again. Raise [Undefined] if the
     forcing of [x] tries to force [x] itself recursively. *)
-external force : 'a t -> 'a = "%lazy_force"
+external force : ('a t[@local_opt]) -> 'a = "%lazy_force"
 
 (** Like [force] except that [force_val x] does not use an exception handler, so it may be
     more efficient.  However, if the computation of [x] raises an exception, it is
@@ -65,6 +62,10 @@ val from_val : 'a -> 'a t
     exception. *)
 val is_val : 'a t -> bool
 
+(** [peek x] returns None if [x] has never been forced or [Some v] if [x] was forced
+    to value [v] *)
+val peek : 'a t -> 'a option
+
 (** This type offers a serialization function [sexp_of_t] that won't force its argument.
     Instead, it will serialize the ['a] if it is available, or just use a custom string
     indicating it is not forced. Note that this is not a round-trippable type, thus the
@@ -73,7 +74,7 @@ val is_val : 'a t -> bool
 module T_unforcing : sig
   type nonrec 'a t = 'a t [@@deriving_inline sexp_of]
 
-  val sexp_of_t : ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t
+  val sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
 
   [@@@end]
 end
