@@ -5,12 +5,12 @@
 
 open! Import
 
-type 'a t [@@deriving_inline sexp]
-include
-  sig
-    [@@@ocaml.warning "-32"]
-    include Ppx_sexp_conv_lib.Sexpable.S1 with type 'a t :=  'a t
-  end[@@ocaml.doc "@inline"]
+type 'a t [@@deriving_inline sexp, sexp_grammar]
+
+include Sexplib0.Sexpable.S1 with type 'a t := 'a t
+
+val t_sexp_grammar : 'a Sexplib0.Sexp_grammar.t -> 'a t Sexplib0.Sexp_grammar.t
+
 [@@@end]
 
 val empty : _ t
@@ -18,10 +18,14 @@ val empty : _ t
 (** Initially filled with all [None] *)
 val create : len:int -> _ t
 
+include Indexed_container.Generic with type ('a, _) t := 'a t and type 'a elt := 'a option
 
-val init_some : int -> f:(int -> 'a) -> 'a t
-val init : int -> f:(int -> 'a option) -> 'a t
-val length : _ t -> int
+val length : (_ t[@local]) -> int
+val init_some : int -> f:((int -> 'a)[@local]) -> 'a t
+val init : int -> f:((int -> 'a option)[@local]) -> 'a t
+val of_array : 'a option array -> 'a t
+val of_array_some : 'a array -> 'a t
+val to_array : 'a t -> 'a option Array.t
 
 (** [get t i] returns the element number [i] of array [t], raising if [i] is outside the
     range 0 to [length t - 1]. *)
@@ -63,6 +67,14 @@ val swap : _ t -> int -> int -> unit
 (** Replaces all the elements of the array with [None]. *)
 val clear : _ t -> unit
 
+(** [map f [|a1; ...; an|]] applies function [f] to [a1], [a2], ..., [an], in order,
+    and builds the option_array [[|f a1; ...; f an|]] with the results returned by [f]. *)
+val map : 'a t -> f:(('a option -> 'b option)[@local]) -> 'b t
+
+(** [map_some t ~f] is like [map], but [None] elements always map to [None] and [Some]
+    always map to [Some]. *)
+val map_some : 'a t -> f:(('a -> 'b)[@local]) -> 'b t
+
 (** Unsafe versions of [set*]. Can cause arbitrary behaviour when used for an
     out-of-bounds array access. *)
 
@@ -80,11 +92,9 @@ val copy : 'a t -> 'a t
 module For_testing : sig
   module Unsafe_cheap_option : sig
     type 'a t [@@deriving_inline sexp]
-    include
-      sig
-        [@@@ocaml.warning "-32"]
-        include Ppx_sexp_conv_lib.Sexpable.S1 with type 'a t :=  'a t
-      end[@@ocaml.doc "@inline"]
+
+    include Sexplib0.Sexpable.S1 with type 'a t := 'a t
+
     [@@@end]
 
     val none : _ t
