@@ -31,16 +31,16 @@
 
 open! Import
 
-type +'a t [@@deriving_inline compare, equal, sexp_of]
+type +'a t [@@deriving_inline sexp_of]
 
-val compare : ('a -> 'a -> int) -> 'a t -> 'a t -> int
-val equal : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool
-val sexp_of_t : ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t
+val sexp_of_t : ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t
 
 [@@@end]
 
-type 'a sequence = 'a t
+type 'a sequence := 'a t
 
+include Ppx_compare_lib.Equal.S1 with type 'a t := 'a t
+include Ppx_compare_lib.Comparable.S1 with type 'a t := 'a t
 include Indexed_container.S1 with type 'a t := 'a t
 include Monad.S with type 'a t := 'a t
 
@@ -63,15 +63,18 @@ val next : 'a t -> ('a * 'a t) option
 module Step : sig
   type ('a, 's) t =
     | Done
-    | Skip of 's
-    | Yield of 'a * 's
+    | Skip of { state : 's }
+    | Yield of
+        { value : 'a
+        ; state : 's
+        }
   [@@deriving_inline sexp_of]
 
   val sexp_of_t
-    :  ('a -> Ppx_sexp_conv_lib.Sexp.t)
-    -> ('s -> Ppx_sexp_conv_lib.Sexp.t)
+    :  ('a -> Sexplib0.Sexp.t)
+    -> ('s -> Sexplib0.Sexp.t)
     -> ('a, 's) t
-    -> Ppx_sexp_conv_lib.Sexp.t
+    -> Sexplib0.Sexp.t
 
   [@@@end]
 end
@@ -107,35 +110,46 @@ val nth_exn : 'a t -> int -> 'a
 
 (** [folding_map] is a version of [map] that threads an accumulator through calls to
     [f]. *)
-val folding_map : 'a t -> init:'b -> f:('b -> 'a -> 'b * 'c) -> 'c t
+val folding_map : 'a t -> init:'acc -> f:('acc -> 'a -> 'acc * 'b) -> 'b t
 
-val folding_mapi : 'a t -> init:'b -> f:(int -> 'b -> 'a -> 'b * 'c) -> 'c t
+val folding_mapi : 'a t -> init:'acc -> f:(int -> 'acc -> 'a -> 'acc * 'b) -> 'b t
 val mapi : 'a t -> f:(int -> 'a -> 'b) -> 'b t
 val filteri : 'a t -> f:(int -> 'a -> bool) -> 'a t
 val filter : 'a t -> f:('a -> bool) -> 'a t
 
-(** [merge t1 t2 ~compare] merges two sorted sequences [t1] and [t2], returning a sorted
-    sequence, all according to [compare].  If two elements are equal, the one from [t1] is
-    preferred.  The behavior is undefined if the inputs aren't sorted. *)
+(** If [t1] and [t2] are each sorted without duplicates, [merge_deduped_and_sorted t1 t2
+    ~compare] merges [t1] and [t2] into a sorted sequence without duplicates. Whenever
+    identical elements are found in both [t1] and [t2], the one from [t1] is used and the
+    one from [t2] is discarded. The behavior is undefined if the inputs aren't sorted or
+    contain duplicates. *)
+val merge_deduped_and_sorted : 'a t -> 'a t -> compare:('a -> 'a -> int) -> 'a t
+
 val merge : 'a t -> 'a t -> compare:('a -> 'a -> int) -> 'a t
+[@@deprecated
+  "[since 2021-07] For identical behavior, use [Sequence.merge_deduped_and_sorted], \
+   but consider using [Sequence.merge_sorted] instead."]
+
+(** If [t1] and [t2] are each sorted, [merge_sorted t1 t2 ~compare] merges [t1] and [t2]
+    into a sorted sequence. Whenever identical elements are found in both [t1] and [t2],
+    the one from [t1] is used first. The behavior is undefined if the inputs aren't
+    sorted. *)
+val merge_sorted : 'a t -> 'a t -> compare:('a -> 'a -> int) -> 'a t
 
 module Merge_with_duplicates_element : sig
   type ('a, 'b) t =
     | Left of 'a
     | Right of 'b
     | Both of 'a * 'b
-  [@@deriving_inline compare, hash, sexp]
+  [@@deriving_inline compare, hash, sexp, sexp_grammar]
 
-  val compare : ('a -> 'a -> int) -> ('b -> 'b -> int) -> ('a, 'b) t -> ('a, 'b) t -> int
+  include Ppx_compare_lib.Comparable.S2 with type ('a, 'b) t := ('a, 'b) t
+  include Ppx_hash_lib.Hashable.S2 with type ('a, 'b) t := ('a, 'b) t
+  include Sexplib0.Sexpable.S2 with type ('a, 'b) t := ('a, 'b) t
 
-  val hash_fold_t
-    :  (Ppx_hash_lib.Std.Hash.state -> 'a -> Ppx_hash_lib.Std.Hash.state)
-    -> (Ppx_hash_lib.Std.Hash.state -> 'b -> Ppx_hash_lib.Std.Hash.state)
-    -> Ppx_hash_lib.Std.Hash.state
-    -> ('a, 'b) t
-    -> Ppx_hash_lib.Std.Hash.state
-
-  include Ppx_sexp_conv_lib.Sexpable.S2 with type ('a, 'b) t := ('a, 'b) t
+  val t_sexp_grammar
+    :  'a Sexplib0.Sexp_grammar.t
+    -> 'b Sexplib0.Sexp_grammar.t
+    -> ('a, 'b) t Sexplib0.Sexp_grammar.t
 
   [@@@end]
 end
@@ -160,10 +174,10 @@ val tl_eagerly_exn : 'a t -> 'a t
 
 (** [find_exn t ~f] returns the first element of [t] that satisfies [f]. It raises if
     there is no such element. *)
-val find_exn : 'a t -> f:('a -> bool) -> 'a
+val find_exn : 'a t -> f:(('a -> bool)[@local]) -> 'a
 
 (** Like [for_all], but passes the index as an argument. *)
-val for_alli : 'a t -> f:(int -> 'a -> bool) -> bool
+val for_alli : 'a t -> f:((int -> 'a -> bool)[@local]) -> bool
 
 (** [append t1 t2] first produces the elements of [t1], then produces the elements of
     [t2]. *)
@@ -206,9 +220,9 @@ val zip_full : 'a t -> 'b t -> [ `Left of 'a | `Both of 'a * 'b | `Right of 'b ]
 
 (** [reduce_exn f [a1; ...; an]] is [f (... (f (f a1 a2) a3) ...) an]. It fails on the
     empty sequence. *)
-val reduce_exn : 'a t -> f:('a -> 'a -> 'a) -> 'a
+val reduce_exn : 'a t -> f:(('a -> 'a -> 'a)[@local]) -> 'a
 
-val reduce : 'a t -> f:('a -> 'a -> 'a) -> 'a option
+val reduce : 'a t -> f:(('a -> 'a -> 'a)[@local]) -> 'a option
 
 (** [group l ~break] returns a sequence of lists (i.e., groups) whose concatenation is
     equal to the original sequence. Each group is broken where [break] returns true on a
@@ -225,7 +239,10 @@ val group : 'a t -> break:('a -> 'a -> bool) -> 'a list t
 (** [find_consecutive_duplicate t ~equal] returns the first pair of consecutive elements
     [(a1, a2)] in [t] such that [equal a1 a2].  They are returned in the same order as
     they appear in [t]. *)
-val find_consecutive_duplicate : 'a t -> equal:('a -> 'a -> bool) -> ('a * 'a) option
+val find_consecutive_duplicate
+  :  'a t
+  -> equal:(('a -> 'a -> bool)[@local])
+  -> ('a * 'a) option
 
 (** The same sequence with consecutive duplicates removed.  The relative order of the
     other elements is unaffected. *)
@@ -290,7 +307,7 @@ val drop_while : 'a t -> f:('a -> bool) -> 'a t
     alternatives would mean forcing the consumer to evaluate the first element again (if
     the previous state of the sequence is returned) or take on extra cost for each element
     (if the element is added to the final state of the sequence using [shift_right]). *)
-val drop_while_option : 'a t -> f:('a -> bool) -> ('a * 'a t) option
+val drop_while_option : 'a t -> f:(('a -> bool)[@local]) -> ('a * 'a t) option
 
 (** [split_n t n] immediately consumes the first [n] elements of [t] and returns the
     consumed prefix, as a list, along with the unevaluated tail of [t]. *)
@@ -408,8 +425,8 @@ val bounded_length : _ t -> at_most:int -> [ `Is of int | `Greater ]
     through only as much of the sequence as necessary. *)
 val length_is_bounded_by : ?min:int -> ?max:int -> _ t -> bool
 
-val of_seq : 'a Caml.Seq.t -> 'a t
-val to_seq : 'a t -> 'a Caml.Seq.t
+val of_seq : 'a Stdlib.Seq.t -> 'a t
+val to_seq : 'a t -> 'a Stdlib.Seq.t
 
 (** [Generator] is a monadic interface to generate sequences in a direct style, similar to
     Python's generators.
