@@ -8,20 +8,19 @@ type t =
   | Zero
   | Pos
   | Nan
-[@@deriving_inline enumerate, hash]
-include
-  sig
-    [@@@ocaml.warning "-32"]
-    val all : t list
-    val hash_fold_t :
-      Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state
-    val hash : t -> Ppx_hash_lib.Std.Hash.hash_value
-  end[@@ocaml.doc "@inline"]
+[@@deriving_inline enumerate, sexp_grammar]
+
+include Ppx_enumerate_lib.Enumerable.S with type t := t
+
+val t_sexp_grammar : t Sexplib0.Sexp_grammar.t
+
 [@@@end]
 
 (** This provides [to_string]/[of_string], sexp conversion, Map, Hashtbl, etc. *)
-include
-  Identifiable.S with type t := t
+include Identifiable.S with type t := t
+
+(** Returns the human-readable strings "positive", "negative", "zero", "not-a-number". *)
+val to_string_hum : t -> string
 
 val of_int : int -> t
 
