@@ -13,19 +13,20 @@ type 'a t =
   }
 [@@deriving_inline sexp_of]
 
-let sexp_of_t : 'a. ('a -> Ppx_sexp_conv_lib.Sexp.t) -> 'a t -> Ppx_sexp_conv_lib.Sexp.t =
-  fun _of_a -> function
-    | { length = v_length; elts = v_elts } ->
-      let bnds = [] in
-      let bnds =
-        let arg = Option_array.sexp_of_t _of_a v_elts in
-        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "elts"; arg ] :: bnds
-      in
-      let bnds =
-        let arg = sexp_of_int v_length in
-        Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "length"; arg ] :: bnds
-      in
-      Ppx_sexp_conv_lib.Sexp.List bnds
+let sexp_of_t : 'a. ('a -> Sexplib0.Sexp.t) -> 'a t -> Sexplib0.Sexp.t =
+  fun _of_a__001_ { length = length__003_; elts = elts__005_ } ->
+  let bnds__002_ = ([] : _ Stdlib.List.t) in
+  let bnds__002_ =
+    let arg__006_ = Option_array.sexp_of_t _of_a__001_ elts__005_ in
+    (Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom "elts"; arg__006_ ] :: bnds__002_
+     : _ Stdlib.List.t)
+  in
+  let bnds__002_ =
+    let arg__004_ = sexp_of_int length__003_ in
+    (Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom "length"; arg__004_ ] :: bnds__002_
+     : _ Stdlib.List.t)
+  in
+  Sexplib0.Sexp.List bnds__002_
 ;;
 
 [@@@end]
@@ -59,7 +60,7 @@ let length t = t.length
 let is_empty t = length t = 0
 
 (* The order in which elements are visited has been chosen so as to be backwards
-   compatible with both [Linked_stack] and [Caml.Stack] *)
+   compatible with [Stdlib.Stack] *)
 let fold t ~init ~f =
   let r = ref init in
   for i = t.length - 1 downto 0 do
@@ -116,6 +117,12 @@ let of_list (type a) (l : a list) =
 let sexp_of_t sexp_of_a t = List.sexp_of_t sexp_of_a (to_list t)
 let t_of_sexp a_of_sexp sexp = of_list (List.t_of_sexp a_of_sexp sexp)
 
+let t_sexp_grammar (type a) (grammar : a Sexplib0.Sexp_grammar.t)
+  : a t Sexplib0.Sexp_grammar.t
+  =
+  Sexplib0.Sexp_grammar.coerce (List.t_sexp_grammar grammar)
+;;
+
 let resize t size =
   let arr = Option_array.create ~len:size in
   Option_array.blit ~src:t.elts ~dst:arr ~src_pos:0 ~dst_pos:0 ~len:t.length;
@@ -166,7 +173,45 @@ let until_empty t f =
       f (pop_nonempty t);
       loop ())
   in
-  loop ()
+  loop () [@nontail]
+;;
+
+let filter_map t ~f =
+  let t_result = create () in
+  for i = 0 to t.length - 1 do
+    match f (Option_array.get_some_exn t.elts i) with
+    | None -> ()
+    | Some x -> push t_result x
+  done;
+  t_result
+;;
+
+let filter t ~f =
+  let t_result = create () in
+  for i = 0 to t.length - 1 do
+    let x = Option_array.get_some_exn t.elts i in
+    if f x then push t_result x
+  done;
+  t_result
+;;
+
+let filter_inplace t ~f =
+  let write_index = ref 0 in
+  Exn.protect
+    ~f:(fun () ->
+      for read_index = 0 to t.length - 1 do
+        let x = Option_array.unsafe_get_some_assuming_some t.elts read_index in
+        if f x
+        then (
+          if !write_index < read_index
+          then Option_array.unsafe_set_some t.elts !write_index x;
+          incr write_index)
+      done)
+    ~finally:(fun () ->
+      for i = !write_index to t.length - 1 do
+        Option_array.unsafe_set_none t.elts i
+      done;
+      t.length <- !write_index) [@nontail]
 ;;
 
 let singleton x =
