@@ -1,6 +1,7 @@
 open! Import
 module Array = Array0
 module Bytes = Bytes0
+module Int = Int0
 include String0
 
 let invalid_argf = Printf.invalid_argf
@@ -8,7 +9,9 @@ let raise_s = Error.raise_s
 let stage = Staged.stage
 
 module T = struct
-  type t = string [@@deriving_inline hash, sexp, sexp_grammar]
+  type t = string [@@deriving_inline globalize, hash, sexp, sexp_grammar]
+
+  let (globalize : (t[@ocaml.local]) -> t) = (globalize_string : (t[@ocaml.local]) -> t)
 
   let (hash_fold_t : Ppx_hash_lib.Std.Hash.state -> t -> Ppx_hash_lib.Std.Hash.state) =
     hash_fold_string
@@ -18,31 +21,13 @@ module T = struct
     fun x -> func x
   ;;
 
-  let t_of_sexp = (string_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
-  let sexp_of_t = (sexp_of_string : t -> Ppx_sexp_conv_lib.Sexp.t)
-
-  let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
-    let (_the_generic_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.generic_group) =
-      { implicit_vars = [ "string" ]
-      ; ggid = "\146e\023\249\235eE\139c\132W\195\137\129\235\025"
-      ; types = [ "t", Implicit_var 0 ]
-      }
-    in
-    let (_the_group : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.group) =
-      { gid = Ppx_sexp_conv_lib.Lazy_group_id.create ()
-      ; apply_implicit = [ string_sexp_grammar ]
-      ; generic_group = _the_generic_group
-      ; origin = "string.ml.T"
-      }
-    in
-    let (t_sexp_grammar : Ppx_sexp_conv_lib.Sexp.Private.Raw_grammar.t) =
-      Ref ("t", _the_group)
-    in
-    t_sexp_grammar
-  ;;
+  let t_of_sexp = (string_of_sexp : Sexplib0.Sexp.t -> t)
+  let sexp_of_t = (sexp_of_string : t -> Sexplib0.Sexp.t)
+  let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = string_sexp_grammar
 
   [@@@end]
 
+  let hashable : t Hashable.t = { hash; compare; sexp_of_t }
   let compare = compare
 end
 
@@ -57,14 +42,21 @@ let invariant (_ : t) = ()
    [sub], [subo] could be implemented using [Blit.Make(Bytes)] plus unsafe casts to/from
    string but were inlined here to avoid using [Bytes.unsafe_of_string] as much as possible.
 *)
+let unsafe_sub src ~pos ~len =
+  if len = 0
+  then ""
+  else (
+    let dst = Bytes.create len in
+    Bytes.unsafe_blit_string ~src ~src_pos:pos ~dst ~dst_pos:0 ~len;
+    Bytes.unsafe_to_string ~no_mutation_while_string_reachable:dst)
+;;
+
 let sub src ~pos ~len =
   if pos = 0 && len = String.length src
   then src
   else (
     Ordered_collection_common.check_pos_len_exn ~pos ~len ~total_length:(length src);
-    let dst = Bytes.create len in
-    if len > 0 then Bytes.unsafe_blit_string ~src ~src_pos:pos ~dst ~dst_pos:0 ~len;
-    Bytes.unsafe_to_string ~no_mutation_while_string_reachable:dst)
+    unsafe_sub src ~pos ~len)
 ;;
 
 let subo ?(pos = 0) ?len src =
@@ -156,22 +148,22 @@ let rindex_from_exn =
 
 let index t char =
   try Some (index_exn t char) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 let rindex t char =
   try Some (rindex_exn t char) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 let index_from t pos char =
   try Some (index_from_exn t pos char) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 let rindex_from t pos char =
   try Some (rindex_from_exn t pos char) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 module Search_pattern0 = struct
@@ -343,12 +335,7 @@ module Search_pattern0 = struct
       let next_src_pos = ref 0 in
       List.iter matches ~f:(fun i ->
         let len = i - !next_src_pos in
-        Bytes.blit_string
-          ~src:s
-          ~src_pos:!next_src_pos
-          ~dst
-          ~dst_pos:!next_dst_pos
-          ~len;
+        Bytes.blit_string ~src:s ~src_pos:!next_src_pos ~dst ~dst_pos:!next_dst_pos ~len;
         Bytes.blit_string
           ~src:with_
           ~src_pos:0
@@ -366,6 +353,15 @@ module Search_pattern0 = struct
       Bytes.unsafe_to_string ~no_mutation_while_string_reachable:dst
   ;;
 
+  let split_on t s =
+    let pattern_len = String.length t.pattern in
+    let matches = index_all t ~may_overlap:false ~in_:s in
+    List.map2_exn
+      (-pattern_len :: matches)
+      (matches @ [ String.length s ])
+      ~f:(fun i j -> sub s ~pos:(i + pattern_len) ~len:(j - i - pattern_len))
+  ;;
+
   module Private = struct
     type public = t
 
@@ -377,43 +373,42 @@ module Search_pattern0 = struct
     [@@deriving_inline equal, sexp_of]
 
     let equal =
-      (fun a__001_ b__002_ ->
-         if Ppx_compare_lib.phys_equal a__001_ b__002_
+      (fun a__003_ b__004_ ->
+         if Stdlib.( == ) a__003_ b__004_
          then true
          else
-           Ppx_compare_lib.( && )
-             (equal_string a__001_.pattern b__002_.pattern)
-             (Ppx_compare_lib.( && )
-                (equal_bool a__001_.case_sensitive b__002_.case_sensitive)
-                (equal_array equal_int a__001_.kmp_array b__002_.kmp_array))
+           Stdlib.( && )
+             (equal_string a__003_.pattern b__004_.pattern)
+             (Stdlib.( && )
+                (equal_bool a__003_.case_sensitive b__004_.case_sensitive)
+                (equal_array equal_int a__003_.kmp_array b__004_.kmp_array))
            : t -> t -> bool)
     ;;
 
     let sexp_of_t =
-      (function
-        | { pattern = v_pattern
-          ; case_sensitive = v_case_sensitive
-          ; kmp_array = v_kmp_array
-          } ->
-          let bnds = [] in
-          let bnds =
-            let arg = sexp_of_array sexp_of_int v_kmp_array in
-            Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "kmp_array"; arg ]
-            :: bnds
-          in
-          let bnds =
-            let arg = sexp_of_bool v_case_sensitive in
-            Ppx_sexp_conv_lib.Sexp.List
-              [ Ppx_sexp_conv_lib.Sexp.Atom "case_sensitive"; arg ]
-            :: bnds
-          in
-          let bnds =
-            let arg = sexp_of_string v_pattern in
-            Ppx_sexp_conv_lib.Sexp.List [ Ppx_sexp_conv_lib.Sexp.Atom "pattern"; arg ]
-            :: bnds
-          in
-          Ppx_sexp_conv_lib.Sexp.List bnds
-          : t -> Ppx_sexp_conv_lib.Sexp.t)
+      (fun { pattern = pattern__008_
+           ; case_sensitive = case_sensitive__010_
+           ; kmp_array = kmp_array__012_
+           } ->
+        let bnds__007_ = ([] : _ Stdlib.List.t) in
+        let bnds__007_ =
+          let arg__013_ = sexp_of_array sexp_of_int kmp_array__012_ in
+          (Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom "kmp_array"; arg__013_ ] :: bnds__007_
+           : _ Stdlib.List.t)
+        in
+        let bnds__007_ =
+          let arg__011_ = sexp_of_bool case_sensitive__010_ in
+          (Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom "case_sensitive"; arg__011_ ]
+           :: bnds__007_
+           : _ Stdlib.List.t)
+        in
+        let bnds__007_ =
+          let arg__009_ = sexp_of_string pattern__008_ in
+          (Sexplib0.Sexp.List [ Sexplib0.Sexp.Atom "pattern"; arg__009_ ] :: bnds__007_
+           : _ Stdlib.List.t)
+        in
+        Sexplib0.Sexp.List bnds__007_
+        : t -> Sexplib0.Sexp.t)
     ;;
 
     [@@@end]
@@ -444,10 +439,7 @@ let substr_index_all_gen ~case_sensitive t ~may_overlap ~pattern =
 ;;
 
 let substr_replace_first_gen ~case_sensitive ?pos t ~pattern =
-  Search_pattern.replace_first
-    ?pos
-    (Search_pattern.create ~case_sensitive pattern)
-    ~in_:t
+  Search_pattern.replace_first ?pos (Search_pattern.create ~case_sensitive pattern) ~in_:t
 ;;
 
 let substr_replace_all_gen ~case_sensitive t ~pattern =
@@ -470,8 +462,7 @@ let is_substring_at_gen =
     if sub_pos = sub_len
     then true
     else if char_equal (unsafe_get str str_pos) (unsafe_get sub sub_pos)
-    then
-      loop ~str ~str_pos:(str_pos + 1) ~sub ~sub_pos:(sub_pos + 1) ~sub_len ~char_equal
+    then loop ~str ~str_pos:(str_pos + 1) ~sub ~sub_pos:(sub_pos + 1) ~sub_len ~char_equal
     else false
   in
   fun str ~pos:str_pos ~substring:sub ~char_equal ->
@@ -508,16 +499,15 @@ let is_prefix_gen string ~prefix ~char_equal =
 
 module Caseless = struct
   module T = struct
-    type t = string [@@deriving_inline sexp]
+    type t = string [@@deriving_inline sexp, sexp_grammar]
 
-    let t_of_sexp = (string_of_sexp : Ppx_sexp_conv_lib.Sexp.t -> t)
-    let sexp_of_t = (sexp_of_string : t -> Ppx_sexp_conv_lib.Sexp.t)
+    let t_of_sexp = (string_of_sexp : Sexplib0.Sexp.t -> t)
+    let sexp_of_t = (sexp_of_string : t -> Sexplib0.Sexp.t)
+    let (t_sexp_grammar : t Sexplib0.Sexp_grammar.t) = string_sexp_grammar
 
     [@@@end]
 
-    let char_compare_caseless c1 c2 =
-      Char.compare (Char.lowercase c1) (Char.lowercase c2)
-    ;;
+    let char_compare_caseless c1 c2 = Char.compare (Char.lowercase c1) (Char.lowercase c2)
 
     let rec compare_loop ~pos ~string1 ~len1 ~string2 ~len2 =
       if pos = len1
@@ -525,9 +515,7 @@ module Caseless = struct
       else if pos = len2
       then 1
       else (
-        let c =
-          char_compare_caseless (unsafe_get string1 pos) (unsafe_get string2 pos)
-        in
+        let c = char_compare_caseless (unsafe_get string1 pos) (unsafe_get string2 pos) in
         match c with
         | 0 -> compare_loop ~pos:(pos + 1) ~string1 ~len1 ~string2 ~len2
         | _ -> c)
@@ -597,7 +585,7 @@ let rev t =
   let len = length t in
   let res = Bytes.create len in
   for i = 0 to len - 1 do
-    unsafe_set res i (unsafe_get t (len - 1 - i))
+    Bytes.unsafe_set res i (unsafe_get t (len - 1 - i))
   done;
   Bytes.unsafe_to_string ~no_mutation_while_string_reachable:res
 ;;
@@ -626,12 +614,12 @@ let rsplit2_exn =
 
 let lsplit2 line ~on =
   try Some (lsplit2_exn line ~on) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 let rsplit2 line ~on =
   try Some (rsplit2_exn line ~on) with
-  | Not_found_s _ | Caml.Not_found -> None
+  | Not_found_s _ | Stdlib.Not_found -> None
 ;;
 
 let rec char_list_mem l (c : char) =
@@ -721,7 +709,7 @@ let suffix t n = wrap_sub_n ~name:"suffix" t n ~pos:(length t - n) ~len:n ~on_er
 let lfindi ?(pos = 0) t ~f =
   let n = length t in
   let rec loop i = if i = n then None else if f i t.[i] then Some i else loop (i + 1) in
-  loop pos
+  loop pos [@nontail]
 ;;
 
 let find t ~f =
@@ -740,7 +728,7 @@ let find_map t ~f =
       | None -> loop (i + 1)
       | Some _ as res -> res)
   in
-  loop 0
+  loop 0 [@nontail]
 ;;
 
 let rfindi ?pos t ~f =
@@ -750,10 +738,10 @@ let rfindi ?pos t ~f =
     | Some pos -> pos
     | None -> length t - 1
   in
-  loop pos
+  loop pos [@nontail]
 ;;
 
-let last_non_drop ~drop t = rfindi t ~f:(fun _ c -> not (drop c))
+let last_non_drop ~drop t = rfindi t ~f:(fun _ c -> not (drop c)) [@nontail]
 
 let rstrip ?(drop = Char.is_whitespace) t =
   match last_non_drop t ~drop with
@@ -761,7 +749,7 @@ let rstrip ?(drop = Char.is_whitespace) t =
   | Some i -> if i = length t - 1 then t else prefix t (i + 1)
 ;;
 
-let first_non_drop ~drop t = lfindi t ~f:(fun _ c -> not (drop c))
+let first_non_drop ~drop t = lfindi t ~f:(fun _ c -> not (drop c)) [@nontail]
 
 let lstrip ?(drop = Char.is_whitespace) t =
   match first_non_drop t ~drop with
@@ -817,16 +805,24 @@ let for_all =
   fun s ~f -> loop s 0 ~len:(length s) ~f
 ;;
 
-let fold t ~init ~f =
-  let n = length t in
-  let rec loop i ac = if i = n then ac else loop (i + 1) (f ac t.[i]) in
-  loop 0 init
+let fold =
+  let rec loop t i ac ~f ~len =
+    if i = len then ac else loop t (i + 1) (f ac t.[i]) ~f ~len
+  in
+  fun t ~init ~f -> loop t 0 init ~f ~len:(length t)
 ;;
 
-let foldi t ~init ~f =
-  let n = length t in
-  let rec loop i ac = if i = n then ac else loop (i + 1) (f i ac t.[i]) in
-  loop 0 init
+let foldi =
+  let rec loop t i ac ~f ~len =
+    if i = len then ac else loop t (i + 1) (f i ac t.[i]) ~f ~len
+  in
+  fun t ~init ~f -> loop t 0 init ~f ~len:(length t)
+;;
+
+let iteri t ~f =
+  for i = 0 to length t - 1 do
+    f i (unsafe_get t i)
+  done
 ;;
 
 let count t ~f = Container.count ~fold t ~f
@@ -834,7 +830,12 @@ let sum m t ~f = Container.sum ~fold m t ~f
 let min_elt t = Container.min_elt ~fold t
 let max_elt t = Container.max_elt ~fold t
 let fold_result t ~init ~f = Container.fold_result ~fold ~init ~f t
-let fold_until t ~init ~f = Container.fold_until ~fold ~init ~f t
+let fold_until t ~init ~f ~finish = Container.fold_until ~fold ~init ~f t ~finish
+let find_mapi t ~f = Indexed_container.find_mapi ~iteri t ~f
+let findi t ~f = Indexed_container.findi ~iteri t ~f
+let counti t ~f = Indexed_container.counti ~foldi t ~f
+let for_alli t ~f = Indexed_container.for_alli ~iteri t ~f
+let existsi t ~f = Indexed_container.existsi ~iteri t ~f
 
 let mem =
   let rec loop t c ~pos:i ~len =
@@ -896,6 +897,46 @@ let tr_multi ~target ~replacement =
 
 let concat_array ?sep ar = concat ?sep (Array.to_list ar)
 let concat_map ?sep s ~f = concat_array ?sep (Array.map (to_array s) ~f)
+let concat_mapi ?sep t ~f = concat_array ?sep (Array.mapi (to_array t) ~f)
+
+let concat_lines =
+  let rec line_lengths ~lines ~newline_len ~sum =
+    match lines with
+    | [] -> sum
+    | line :: lines ->
+      let sum = sum + String.length line + newline_len in
+      line_lengths ~lines ~newline_len ~sum
+  in
+  let rec write_lines ~buf ~lines ~crlf ~pos =
+    match lines with
+    | [] -> pos
+    | line :: lines ->
+      Bytes.unsafe_blit_string
+        ~src:line
+        ~src_pos:0
+        ~dst:buf
+        ~dst_pos:pos
+        ~len:(String.length line);
+      let pos = pos + String.length line in
+      let pos =
+        if crlf
+        then (
+          Bytes.unsafe_set buf pos '\r';
+          pos + 1)
+        else pos
+      in
+      Bytes.unsafe_set buf pos '\n';
+      let pos = pos + 1 in
+      write_lines ~buf ~lines ~crlf ~pos
+  in
+  fun ?(crlf = false) lines ->
+    let newline_len = if crlf then 2 else 1 in
+    let len = line_lengths ~newline_len ~lines ~sum:0 in
+    let buf = Bytes.create len in
+    let written = write_lines ~buf ~lines ~crlf ~pos:0 in
+    assert (written = len);
+    Bytes.unsafe_to_string ~no_mutation_while_string_reachable:buf
+;;
 
 (* [filter t f] is implemented by the following algorithm.
 
@@ -944,6 +985,32 @@ let filter t ~f =
     if !out_pos = n - 1 then out else sub out ~pos:0 ~len:!out_pos)
 ;;
 
+(* repeated code to avoid requiring an extra allocation for a closure on each call. *)
+let filteri t ~f =
+  let n = length t in
+  let i = ref 0 in
+  while !i < n && f !i t.[!i] do
+    incr i
+  done;
+  if !i = n
+  then t
+  else (
+    let out = Bytes.create (n - 1) in
+    Bytes.blit_string ~src:t ~src_pos:0 ~dst:out ~dst_pos:0 ~len:!i;
+    let out_pos = ref !i in
+    incr i;
+    while !i < n do
+      let c = t.[!i] in
+      if f !i c
+      then (
+        Bytes.set out !out_pos c;
+        incr out_pos);
+      incr i
+    done;
+    let out = Bytes.unsafe_to_string ~no_mutation_while_string_reachable:out in
+    if !out_pos = n - 1 then out else sub out ~pos:0 ~len:!out_pos)
+;;
+
 let chop_prefix s ~prefix =
   if is_prefix s ~prefix then Some (drop_prefix s (length prefix)) else None
 ;;
@@ -972,6 +1039,114 @@ let chop_suffix_exn s ~suffix =
   | None -> invalid_argf "String.chop_suffix_exn %S %S" s suffix ()
 ;;
 
+module For_common_prefix_and_suffix = struct
+  (* When taking a string prefix or suffix, we extract from the shortest input available
+     in case we can just return one of our inputs without allocating a new string. *)
+
+  let shorter a b = if length a <= length b then a else b
+
+  let shortest list =
+    match list with
+    | [] -> ""
+    | first :: rest -> List.fold rest ~init:first ~f:shorter
+  ;;
+
+  (* Our generic accessors for common prefix/suffix abstract over [get_pos], which is
+     either [pos_from_left] or [pos_from_right]. *)
+
+  let pos_from_left (_ : t) (i : int) = i
+  let pos_from_right t i = length t - i - 1
+
+  let rec common_generic2_length_loop a b ~get_pos ~max_len ~len_so_far =
+    if len_so_far >= max_len
+    then max_len
+    else if Char.equal
+              (unsafe_get a (get_pos a len_so_far))
+              (unsafe_get b (get_pos b len_so_far))
+    then common_generic2_length_loop a b ~get_pos ~max_len ~len_so_far:(len_so_far + 1)
+    else len_so_far
+  ;;
+
+  let common_generic2_length a b ~get_pos =
+    let max_len = min (length a) (length b) in
+    common_generic2_length_loop a b ~get_pos ~max_len ~len_so_far:0
+  ;;
+
+  let rec common_generic_length_loop first list ~get_pos ~max_len =
+    match list with
+    | [] -> max_len
+    | second :: rest ->
+      let max_len =
+        (* We call [common_generic2_length_loop] rather than [common_generic2_length] so
+           that [max_len] limits our traversal of [first] and [second]. *)
+        common_generic2_length_loop first second ~get_pos ~max_len ~len_so_far:0
+      in
+      common_generic_length_loop second rest ~get_pos ~max_len
+  ;;
+
+  let common_generic_length list ~get_pos =
+    match list with
+    | [] -> 0
+    | first :: rest ->
+      (* Precomputing [max_len] based on [shortest list] saves us work in longer strings,
+         at the cost of an extra pass over the spine of [list].
+
+         For example, if you're looking for the longest prefix of the strings:
+
+         {v
+            let long_a = List.init 1000 ~f:(Fn.const 'a')
+            [ long_a; long_a; 'aa' ]
+         v}
+
+         the approach below will just check the first two characters of all the strings.
+      *)
+      let max_len = length (shortest list) in
+      common_generic_length_loop first rest ~get_pos ~max_len
+  ;;
+
+  (* Our generic accessors that produce a string abstract over [take], which is either
+     [prefix] or [suffix]. *)
+
+  let common_generic2 a b ~get_pos ~take =
+    let len = common_generic2_length a b ~get_pos in
+    (* Use the shorter of the two strings, so that if the shorter one is the shared
+       prefix, [take] won't allocate another string. *)
+    take (shorter a b) len
+  ;;
+
+  let common_generic list ~get_pos ~take =
+    match list with
+    | [] -> ""
+    | first :: rest ->
+      (* As with [common_generic_length], we base [max_len] on [shortest list]. We also
+         use this result for [take], below, to potentially avoid allocating a string. *)
+      let s = shortest list in
+      let max_len = length s in
+      if max_len = 0
+      then ""
+      else (
+        let len =
+          (* We call directly into [common_generic_length_loop] rather than
+             [common_generic_length] to avoid recomputing [shortest list]. *)
+          common_generic_length_loop first rest ~get_pos ~max_len
+        in
+        take s len)
+  ;;
+end
+
+include struct
+  open For_common_prefix_and_suffix
+
+  let common_prefix list = common_generic list ~take:prefix ~get_pos:pos_from_left
+  let common_suffix list = common_generic list ~take:suffix ~get_pos:pos_from_right
+  let common_prefix2 a b = common_generic2 a b ~take:prefix ~get_pos:pos_from_left
+  let common_suffix2 a b = common_generic2 a b ~take:suffix ~get_pos:pos_from_right
+  let common_prefix_length list = common_generic_length list ~get_pos:pos_from_left
+  let common_suffix_length list = common_generic_length list ~get_pos:pos_from_right
+  let common_prefix2_length a b = common_generic2_length a b ~get_pos:pos_from_left
+  let common_suffix2_length a b = common_generic2_length a b ~get_pos:pos_from_right
+end
+
 (* There used to be a custom implementation that was faster for very short strings
    (peaking at 40% faster for 4-6 char long strings).
    This new function is around 20% faster than the default hash function, but slower
@@ -990,11 +1165,10 @@ end
 let _ = hash
 
 include Hash
-include Comparable.Validate (T)
 
 (* for interactive top-levels -- modules deriving from String should have String's pretty
    printer. *)
-let pp = Caml.Format.pp_print_string
+let pp ppf string = Stdlib.Format.fprintf ppf "%S" string
 let of_char c = make 1 c
 
 let of_char_list l =
@@ -1003,6 +1177,218 @@ let of_char_list l =
   Bytes.unsafe_to_string ~no_mutation_while_string_reachable:t
 ;;
 
+let of_list = of_char_list
+let of_array a = init (Array.length a) ~f:(Array.get a)
+let append = ( ^ )
+
+let pad_right ?(char = ' ') s ~len =
+  let src_len = length s in
+  if src_len >= len
+  then s
+  else (
+    let res = Bytes.create len in
+    Bytes.blit_string ~src:s ~dst:res ~src_pos:0 ~dst_pos:0 ~len:src_len;
+    Bytes.fill ~pos:src_len ~len:(len - src_len) res char;
+    Bytes.unsafe_to_string ~no_mutation_while_string_reachable:res)
+;;
+
+let pad_left ?(char = ' ') s ~len =
+  let src_len = length s in
+  if src_len >= len
+  then s
+  else (
+    let res = Bytes.create len in
+    Bytes.blit_string ~src:s ~dst:res ~src_pos:0 ~dst_pos:(len - src_len) ~len:src_len;
+    Bytes.fill ~pos:0 ~len:(len - src_len) res char;
+    Bytes.unsafe_to_string ~no_mutation_while_string_reachable:res)
+;;
+
+(* Called upon first difference generated by filtering. Allocates [buffer_len] bytes
+   for new result, and copies [prefix_len] unchanged characters from [src].
+   Always returns a local buffer. *)
+let local_copy_prefix (src [@local]) ~prefix_len ~buffer_len =
+  let dst = Bytes.create_local buffer_len in
+  Bytes.Primitives.unsafe_blit_string ~src ~dst ~src_pos:0 ~dst_pos:0 ~len:prefix_len;
+   dst
+;;
+
+(* Copies a perhaps-local buffer into a definitely-global string. *)
+let local_copy_to_string (buf [@local]) ~pos =
+  let str = Bytes.unsafe_to_string ~no_mutation_while_string_reachable:buf in
+  unsafe_sub str ~pos:0 ~len:pos [@nontail]
+;;
+
+include struct
+  open struct
+    (* filter_map helpers *)
+
+    (* Filters from string [src] into an allocated buffer [dst];
+       copies the allocated buffer to a heap-allocated result string.
+
+       Pre-conditions:
+       [src_len = length src]
+       [src != dst]
+       [0 <= src_pos < src_len]
+       [0 <= dst_pos < length dst]
+    *)
+    let filter_mapi_into src (dst [@local]) ~f ~src_pos ~dst_pos ~src_len =
+      let dst_pos =  (ref dst_pos) in
+      for src_pos = src_pos to src_len - 1 do
+        match f src_pos (unsafe_get src src_pos) with
+        | None -> ()
+        | Some c ->
+          Bytes.unsafe_set dst !dst_pos c;
+          incr dst_pos
+      done;
+      local_copy_to_string dst ~pos:!dst_pos
+    ;;
+
+    (* Filters [t]. If the result turns out to be identical to the input, returns [t]
+       directly without needing to allocate a buffer and traverse the string twice.
+
+       Pre-condition: [len == length t]
+       Pre-condition: [0 <= pos <= len] *)
+    let rec filter_mapi_maybe_id t ~f ~pos ~len =
+      if pos = len
+      then t
+      else (
+        let c1 = unsafe_get t pos in
+        let next = Int.succ pos in
+        match f pos c1 with
+        | Some c2 when Char.equal c1 c2 ->
+          (* if nothing has changed, continue *)
+          filter_mapi_maybe_id t ~f ~pos:next ~len
+        | option ->
+          (* If a character has been changed or dropped, begin an output buffer up to
+             [pos], and write the new character into it. *)
+          let copy = local_copy_prefix t ~prefix_len:pos ~buffer_len:len in
+          let dst_pos =
+            match option with
+            | None -> pos
+            | Some c ->
+              Bytes.unsafe_set copy pos c;
+              next
+          in
+          filter_mapi_into t copy ~f ~src_pos:next ~dst_pos ~src_len:len [@nontail])
+    ;;
+  end
+
+  (* filter_map functions *)
+
+  let filter_mapi t ~f = filter_mapi_maybe_id t ~f ~pos:0 ~len:(length t)
+  let filter_map t ~f = filter_mapi t ~f:(fun _ c -> f c) [@nontail]
+end
+
+include struct
+  open struct
+    (* partition helpers *)
+
+    let partition_map_into src ~fsts ~snds ~f ~len ~src_pos ~fst_pos ~snd_pos =
+      let fst_pos =  (ref fst_pos) in
+      let snd_pos =  (ref snd_pos) in
+      for src_pos = src_pos to len - 1 do
+        match  (f (unsafe_get src src_pos) : (_, _) Either.t) with
+        | First c ->
+          Bytes.unsafe_set fsts !fst_pos c;
+          incr fst_pos
+        | Second c ->
+          Bytes.unsafe_set snds !snd_pos c;
+          incr snd_pos
+      done;
+      local_copy_to_string fsts ~pos:!fst_pos, local_copy_to_string snds ~pos:!snd_pos
+    ;;
+
+    let partition_map_difference src ~f ~len ~pos:src_pos ~fst_pos ~snd_pos either =
+      let fsts = local_copy_prefix src ~prefix_len:fst_pos ~buffer_len:len in
+      let snds = local_copy_prefix src ~prefix_len:snd_pos ~buffer_len:len in
+      let fst_pos, snd_pos =
+        match (either : (_, _) Either.t) with
+        | First c ->
+          Bytes.unsafe_set fsts fst_pos c;
+           (fst_pos + 1, snd_pos)
+        | Second c ->
+          Bytes.unsafe_set snds snd_pos c;
+           (fst_pos, snd_pos + 1)
+      in
+      partition_map_into
+        src
+        ~fsts
+        ~snds
+        ~f
+        ~len
+        ~src_pos:(src_pos + 1)
+        ~fst_pos
+        ~snd_pos [@nontail]
+    ;;
+
+    let rec partition_map_first_maybe_id src ~f ~pos ~len =
+      if pos = len
+      then src, ""
+      else (
+        let c1 = unsafe_get src pos in
+        match  (f c1 : (_, _) Either.t) with
+        | First c2 when Char.equal c1 c2 ->
+          partition_map_first_maybe_id src ~f ~len ~pos:(pos + 1)
+        | either ->
+          partition_map_difference
+            src
+            ~f
+            ~len
+            ~pos
+            ~fst_pos:pos
+            ~snd_pos:0
+            either [@nontail])
+    ;;
+
+    let rec partition_map_second_maybe_id src ~f ~pos ~len =
+      if pos = len
+      then "", src
+      else (
+        let c1 = unsafe_get src pos in
+        match  (f c1 : (_, _) Either.t) with
+        | Second c2 when Char.equal c1 c2 ->
+          partition_map_second_maybe_id src ~f ~len ~pos:(pos + 1)
+        | either ->
+          partition_map_difference
+            src
+            ~f
+            ~len
+            ~pos
+            ~fst_pos:0
+            ~snd_pos:pos
+            either [@nontail])
+    ;;
+  end
+
+  (* partition functions *)
+
+  let partition_map src ~f =
+    let len = length src in
+    if len = 0
+    then "", ""
+    else (
+      let c1 = unsafe_get src 0 in
+      match  (f c1 : (_, _) Either.t) with
+      | First c2 when Char.equal c1 c2 -> partition_map_first_maybe_id src ~f ~len ~pos:1
+      | Second c2 when Char.equal c1 c2 ->
+        partition_map_second_maybe_id src ~f ~len ~pos:1
+      | either ->
+        partition_map_difference
+          src
+          ~f
+          ~len
+          ~pos:0
+          ~fst_pos:0
+          ~snd_pos:0
+          either [@nontail])
+  ;;
+
+  let partition_tf t ~f =
+    partition_map t ~f:(fun c -> if f c then  (First c) else  (Second c)) [@nontail
+    ]
+  ;;
+end
+
 module Escaping = struct
   (* If this is changed, make sure to update [escape], which attempts to ensure all the
      invariants checked here.  *)
@@ -1258,8 +1644,7 @@ module Escaping = struct
   ;;
 
   let check_bound str pos function_name =
-    if pos >= length str || pos < 0
-    then invalid_argf "%s: out of bounds" function_name ()
+    if pos >= length str || pos < 0 then invalid_argf "%s: out of bounds" function_name ()
   ;;
 
   let is_char_escaping str ~escape_char pos =
@@ -1409,14 +1794,14 @@ module Escaping = struct
     rfindi t ~f:(fun i c ->
       (not (drop c))
       || is_char_escaping t ~escape_char i
-      || is_char_escaped t ~escape_char i)
+      || is_char_escaped t ~escape_char i) [@nontail]
   ;;
 
   let first_non_drop_literal ~drop ~escape_char t =
     lfindi t ~f:(fun i c ->
       (not (drop c))
       || is_char_escaping t ~escape_char i
-      || is_char_escaped t ~escape_char i)
+      || is_char_escaped t ~escape_char i) [@nontail]
   ;;
 
   let rstrip_literal ?(drop = Char.is_whitespace) t ~escape_char =
