@@ -21,43 +21,34 @@ let%test_module _ =
 
      module For_quickcheck = struct
        module Key = struct
-         include Quickcheck.Int
+         include Int
 
-         let quickcheck_generator = Quickcheck.Generator.small_non_negative_int
+         type t = int [@@deriving quickcheck]
+
+         let quickcheck_generator =
+           Base_quickcheck.Generator.small_positive_or_zero_int
+         ;;
        end
 
        module Data = struct
-         include Quickcheck.String
+         include String
 
-         let quickcheck_generator = gen' Quickcheck.Char.gen_lowercase
+         type t = string [@@deriving quickcheck]
+
+         let quickcheck_generator =
+           Base_quickcheck.Generator.string_of
+             Base_quickcheck.Generator.char_lowercase
+         ;;
        end
 
        let compare = Key.compare
 
-       open Quickcheck
-       open Generator
-
        module Constructor = struct
          type t =
            | Add of Key.t * Data.t
            | Replace of Key.t * Data.t
            | Remove of Key.t
-         [@@deriving sexp_of]
-
-         let add_gen =
-           Key.quickcheck_generator
-           >>= fun key ->
-           Data.quickcheck_generator >>| fun data -> Add (key, data)
-         ;;
-
-         let replace_gen =
-           Key.quickcheck_generator
-           >>= fun key ->
-           Data.quickcheck_generator >>| fun data -> Replace (key, data)
-         ;;
-
-         let remove_gen = Key.quickcheck_generator >>| fun key -> Remove key
-         let quickcheck_generator = union [ add_gen; replace_gen; remove_gen ]
+         [@@deriving quickcheck, sexp_of]
 
          let apply_to_tree t tree =
            match t with
@@ -77,14 +68,14 @@ let%test_module _ =
          ;;
        end
 
-       let constructors_gen =
-         List.quickcheck_generator Constructor.quickcheck_generator
-       ;;
+       module Constructors = struct
+         type t = Constructor.t list [@@deriving quickcheck, sexp_of]
+       end
 
        let reify constructors =
          List.fold
            constructors
-           ~init:(empty, Key.Map.empty)
+           ~init:(empty, Map.empty (module Key))
            ~f:(fun (t, map) constructor ->
              ( Constructor.apply_to_tree constructor t
              , Constructor.apply_to_map constructor map ))
@@ -104,11 +95,11 @@ let%test_module _ =
        ;;
 
        let rec to_map = function
-         | Empty -> Key.Map.empty
-         | Leaf { key; value = data } -> Key.Map.singleton key data
+         | Empty -> Map.empty (module Key)
+         | Leaf { key; value = data } -> Map.singleton (module Key) key data
          | Node { left; key; value = data; height = _; right } ->
            merge
-             (Key.Map.singleton key data)
+             (Map.singleton (module Key) key data)
              (merge (to_map left) (to_map right))
        ;;
      end
@@ -124,13 +115,11 @@ let%test_module _ =
      ;;
 
      let is_empty = is_empty
-
      let%test _ = is_empty empty
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: bool] (is_empty t) ~expect:(Map.is_empty map))
@@ -139,25 +128,22 @@ let%test_module _ =
      let invariant = invariant
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            invariant t ~compare;
-           [%test_result: Data.t Key.Map.t] (to_map t) ~expect:map)
+           [%test_result: Data.t Map.M(Key).t] (to_map t) ~expect:map)
      ;;
 
      let add = add
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple4
-            constructors_gen
-            Key.quickcheck_generator
-            Data.quickcheck_generator
-            Quickcheck.Bool.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t * Data.t * bool]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructor.t list * Key.t * Data.t * bool
+           [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key, data, replace) ->
            let t, map = reify constructors in
            (* test [added], other aspects of [add] are tested via [reify] in the
@@ -172,9 +158,10 @@ let%test_module _ =
      let remove = remove
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple2 constructors_gen Key.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key) ->
            let t, map = reify constructors in
            (* test [removed], other aspects of [remove] are tested via [reify] in the
@@ -187,9 +174,10 @@ let%test_module _ =
      let find = find
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple2 constructors_gen Key.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key) ->
            let t, map = reify constructors in
            [%test_result: Data.t option]
@@ -200,9 +188,10 @@ let%test_module _ =
      let mem = mem
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple2 constructors_gen Key.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key) ->
            let t, map = reify constructors in
            [%test_result: bool] (mem t key ~compare) ~expect:(Map.mem map key))
@@ -211,9 +200,8 @@ let%test_module _ =
      let first = first
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: (Key.t * Data.t) option]
@@ -224,9 +212,8 @@ let%test_module _ =
      let last = last
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: (Key.t * Data.t) option]
@@ -237,9 +224,10 @@ let%test_module _ =
      let find_and_call = find_and_call
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple2 constructors_gen Key.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key) ->
            let t, map = reify constructors in
            [%test_result: [ `Found of Data.t | `Not_found of Key.t ]]
@@ -258,9 +246,10 @@ let%test_module _ =
      let findi_and_call = findi_and_call
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple2 constructors_gen Key.quickcheck_generator)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t]
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t [@@deriving quickcheck, sexp_of]
+         end)
          ~f:(fun (constructors, key) ->
            let t, map = reify constructors in
            [%test_result: [ `Found of Key.t * Data.t | `Not_found of Key.t ]]
@@ -279,13 +268,12 @@ let%test_module _ =
      let find_and_call1 = find_and_call1
 
      let%test_unit _ =
-       Quickcheck.test
-         (Quickcheck.Generator.tuple3
-            constructors_gen
-            Key.quickcheck_generator
-            Base_quickcheck.Generator.int)
-         ~sexp_of:[%sexp_of: Constructor.t list * Key.t * int]
-         ~f:(fun (constructors, key, arg) ->
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t * int
+           [@@deriving quickcheck, sexp_of]
+         end)
+         ~f:(fun (constructors, key, a) ->
            let t, map = reify constructors in
            [%test_result:
              [ `Found of Data.t * int | `Not_found of Key.t * int ]]
@@ -293,21 +281,101 @@ let%test_module _ =
                 t
                 key
                 ~compare
-                arg
-                ~if_found:(fun data arg -> `Found (data, arg))
-                ~if_not_found:(fun key arg -> `Not_found (key, arg)))
+                ~a
+                ~if_found:(fun data a -> `Found (data, a))
+                ~if_not_found:(fun key a -> `Not_found (key, a)))
              ~expect:
                (match Map.find map key with
-                | None -> `Not_found (key, arg)
-                | Some data -> `Found (data, arg)))
+                | None -> `Not_found (key, a)
+                | Some data -> `Found (data, a)))
+     ;;
+
+     let findi_and_call1 = findi_and_call1
+
+     let%test_unit _ =
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t * int
+           [@@deriving quickcheck, sexp_of]
+         end)
+         ~f:(fun (constructors, key, a) ->
+           let t, map = reify constructors in
+           [%test_result:
+             [ `Found of Key.t * Data.t * int | `Not_found of Key.t * int ]]
+             (findi_and_call1
+                t
+                key
+                ~compare
+                ~a
+                ~if_found:(fun ~key ~data a -> `Found (key, data, a))
+                ~if_not_found:(fun key a -> `Not_found (key, a)))
+             ~expect:
+               (match Map.find map key with
+                | None -> `Not_found (key, a)
+                | Some data -> `Found (key, data, a)))
+     ;;
+
+     let find_and_call2 = find_and_call2
+
+     let%test_unit _ =
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t * int * string
+           [@@deriving quickcheck, sexp_of]
+         end)
+         ~f:(fun (constructors, key, a, b) ->
+           let t, map = reify constructors in
+           [%test_result:
+             [ `Found of Data.t * int * string
+             | `Not_found of Key.t * int * string
+             ]]
+             (find_and_call2
+                t
+                key
+                ~compare
+                ~a
+                ~b
+                ~if_found:(fun data a b -> `Found (data, a, b))
+                ~if_not_found:(fun key a b -> `Not_found (key, a, b)))
+             ~expect:
+               (match Map.find map key with
+                | None -> `Not_found (key, a, b)
+                | Some data -> `Found (data, a, b)))
+     ;;
+
+     let findi_and_call2 = findi_and_call2
+
+     let%test_unit _ =
+       Base_quickcheck.Test.run_exn
+         (module struct
+           type t = Constructors.t * Key.t * int * string
+           [@@deriving quickcheck, sexp_of]
+         end)
+         ~f:(fun (constructors, key, a, b) ->
+           let t, map = reify constructors in
+           [%test_result:
+             [ `Found of Key.t * Data.t * int * string
+             | `Not_found of Key.t * int * string
+             ]]
+             (findi_and_call2
+                t
+                key
+                ~compare
+                ~a
+                ~b
+                ~if_found:(fun ~key ~data a b -> `Found (key, data, a, b))
+                ~if_not_found:(fun key a b -> `Not_found (key, a, b)))
+             ~expect:
+               (match Map.find map key with
+                | None -> `Not_found (key, a, b)
+                | Some data -> `Found (key, data, a, b)))
      ;;
 
      let iter = iter
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: (Key.t * Data.t) list]
@@ -320,9 +388,8 @@ let%test_module _ =
      let mapi_inplace = mapi_inplace
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: (Key.t * Data.t) list]
@@ -337,9 +404,8 @@ let%test_module _ =
      let fold = fold
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: (Key.t * Data.t) list]
@@ -350,9 +416,8 @@ let%test_module _ =
      let choose_exn = choose_exn
 
      let%test_unit _ =
-       Quickcheck.test
-         constructors_gen
-         ~sexp_of:[%sexp_of: Constructor.t list]
+       Base_quickcheck.Test.run_exn
+         (module Constructors)
          ~f:(fun constructors ->
            let t, map = reify constructors in
            [%test_result: bool]
