@@ -6,8 +6,7 @@ let%expect_test _ =
   let s63_emul = Int63_emul.(Hex.to_string min_value) in
   print_s [%message (s63 : string) (s63_emul : string)];
   require [%here] (String.equal s63 s63_emul);
-  [%expect
-    {|
+  [%expect {|
     ((s63      -0x4000000000000000)
      (s63_emul -0x4000000000000000)) |}]
 ;;
